;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x06755010af3ee59a11784f9a8290d984 . "./code.cg2js/cg-acache.fasl")
(#xe5d1db62f779053ca018bdb1cfe99b58 . "./code.cg2js/cg-base.fasl")
(#xdf09faad6440d620eb650e0de35a3601 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x73e9a67b272b6ca431dedb709d81c9e6 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x644ee306aa5f2dfd4161b014c986d4e3 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xd98aae8677457d0ee216a8501bd79189 . "./code.cg2js/cg-button.fasl")
(#xe175d89794b7bd5993677ce86f4623ca . "./code.cg2js/cg-calendar.fasl")
(#x2844b77c1f789529dc6af995cce6d943 . "./code.cg2js/cg-caret.fasl")
(#x04eae5aa81a6c0713bd472bd0a998ca5 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x29680f09a412b54ea09b69f01bb179df . "./code.cg2js/cg-chart-widget.fasl")
(#xcb523231dfe3b715e6e35af389d2dba6 . "./code.cg2js/cg-check-box.fasl")
(#x7f0d030c4807337505141d3e9347387a . "./code.cg2js/cg-choice-list.fasl")
(#xfd827f197ed96fcb851da13da261dc1e . "./code.cg2js/cg-choose-printer.fasl")
(#xc7b3eacd0f6e89546ac09127cfe9204d . "./code.cg2js/cg-class-grid.fasl")
(#x5dc12bbbc46e5182f1befef3e99541c2 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x495b278f3ffe05853ef23c745fac1475 . "./code.cg2js/cg-class-support.fasl")
(#x11bdb09992432400de4c0a74475e6d65 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x0686d035d2d08bb591c1ec9aa74d2ee9 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x233a23cf9ad3538fdef3354131dc5fa3 . "./code.cg2js/cg-clipboard.fasl")
(#xba41b331101fac5cadd3d20912df9486 . "./code.cg2js/cg-color-dialog.fasl")
(#x269eb14f37d2975accc33fc8ca4e769d . "./code.cg2js/cg-combo-box.fasl")
(#xedd1fa153ab7bd10c14d73237844b2bc . "./code.cg2js/cg-common-control.fasl")
(#x492cf095cfe909adbf3de5255b2c9be8 . "./code.cg2js/cg-comtab.fasl")
(#x7673b658d9db2c0534a4ea6fe1a7fbcd . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xea4337de651daceab9e89f3238c30233 . "./code.cg2js/cg-curve.fasl")
(#xeed6bcf31589441d08bc595f9bd1754b . "./code.cg2js/cg-dde-utils.fasl")
(#x104d8521815c564c92d98922f5cd24f9 . "./code.cg2js/cg-dev.fasl")
(#xe91dd409ac718d5e271c71e4f229ff69 . "./code.cg2js/cg-dialog-item.fasl")
(#x625f80579b79c398d330d678fa0b028e . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x8e7020346401349510c4fc9d6822faad . "./code.cg2js/cg-directory-dialog.fasl")
(#x56e6c2f9f6467d172cc80c7db434bb58 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x586606cc256ad674e0b30179f58ee89a . "./code.cg2js/cg-drag-and-drop.fasl")
(#x7e117c95f4bf841695f3beaa6fbee081 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x1ac7d10abcab72501a9dc44f47a007c0 . "./code.cg2js/cg-drawable.fasl")
(#x4a3d3e8c637ad727e046d837fb85bfff . "./code.cg2js/cg-dropping-outline.fasl")
(#xe5174cb7748419e9ff6497098a320940 . "./code.cg2js/cg-edit-in-place.fasl")
(#xa195470b81a0b21672e2384092f345c7 . "./code.cg2js/cg-editable-text.fasl")
(#xa34a290806338d102b45bf9172403c61 . "./code.cg2js/cg-file-dialog.fasl")
(#x3e66aa21adddfbe284a7dee0add87b8d . "./code.cg2js/cg-fill-texture.fasl")
(#x1358c4a250fd94ef1e22007c19d6a4d2 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x696cc22a6c4b177ea1e92399fcddd83e . "./code.cg2js/cg-font-dialog.fasl")
(#x165d607574aad5279796f94567d66db9 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x4dcaf9bdcf81bd697f78c074d7ea1cc8 . "./code.cg2js/cg-get-pixmap.fasl")
(#x4f62eab6356d070b763de3917410db77 . "./code.cg2js/cg-get-position.fasl")
(#xa257f4ae66c7d095e0a3376dbd5776ce . "./code.cg2js/cg-graphics-context.fasl")
(#x960f4cd27d94d594ce4fee328d95a200 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x4d094d72b779614d172e58d85ca1edee . "./code.cg2js/cg-grid-widget.fasl")
(#x98fccc432c3215cd0bf72639f1f12690 . "./code.cg2js/cg-group-box.fasl")
(#x9ab9be62c2ab751d8ef918a5e4957b56 . "./code.cg2js/cg-header-control.fasl")
(#xd4ba61d9a0d38e30972ec3c4404fc35d . "./code.cg2js/cg-hotspot.fasl")
(#xb197a09bdd20e714946b3ee0e05df1f9 . "./code.cg2js/cg-html-dialog.fasl")
(#xc67925854641fe5d7f41d31560fb6fd5 . "./code.cg2js/cg-html-widget.fasl")
(#xc35b147e3ee8e3170d2231af24e96d09 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x6a0ae6040f0c77da936374f6e7b76638 . "./code.cg2js/cg-icon.fasl")
(#x1b81b8825f5b449545414c16f1b4a2ba . "./code.cg2js/cg-ie.fasl")
(#xfc79c290218a550b6fee80fb32fd73d5 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x51c2a99d061c3213422b05ada988d614 . "./code.cg2js/cg-intersect.fasl")
(#x1ac04391bc5a6d092ebdfdbc28f53f12 . "./code.cg2js/cg-item-list.fasl")
(#xba9102356ea3a4f9372419a12eb2aaea . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x840b26a133d8fd280f0d1ee06d55e2e7 . "./code.cg2js/cg-lamp.fasl")
(#x62ae8c2a2c8565d95f6dfc315fbf8883 . "./code.cg2js/cg-layout.fasl")
(#xc8bc4ea3cca0d76266b7c95f151665a0 . "./code.cg2js/cg-lettered-menu.fasl")
(#x238a254191da786c85a3bb7f851596dc . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x1ed17246573075a6a351c39a71e14744 . "./code.cg2js/cg-lisp-text.fasl")
(#x82baace93406a1ae403933af03648906 . "./code.cg2js/cg-lisp-widget.fasl")
(#x4fc2fd5131efac66c93c44337d869494 . "./code.cg2js/cg-list-view.fasl")
(#xb8e027744ea8d8fdb31a80f8159686ad . "./code.cg2js/cg-mci.fasl")
(#xcfbd22a328b8f74af210af47c78fe726 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x779ebd65a3eac22a43f1b93d620a3a0d . "./code.cg2js/cg-menu.fasl")
(#x0e07765bcb6eb7bbc243435498f16182 . "./code.cg2js/cg-message-dialog.fasl")
(#xc5f256d45aaa16420dd2acf438f511a4 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xaf5766f3e10b3d664054e694010db5f1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5c1dcde1d175ff5f96208eb15e10f479 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x5bd401bcc54f8214f34c0fc3449f7ea4 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xb89dfe3242fdce113c8abb5fac0d6426 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x37066dfdc74819341b4afa44e69f8297 . "./code.cg2js/cg-nodes.fasl")
(#x8d2ba6fb21ea23af9ec8e67af417911d . "./code.cg2js/cg-object-editor-layout.fasl")
(#x5e229219a3706847efef786daa960d7a . "./code.cg2js/cg-object-editor.fasl")
(#x019eda34056fd9ce50f7ce624668fa30 . "./code.cg2js/cg-ocx.fasl")
(#x5c920d9f8e69463140286ace20db568b . "./code.cg2js/cg-os-widget.fasl")
(#x9b3cb97cf95ee8de6f29b82f70d7d847 . "./code.cg2js/cg-os-window.fasl")
(#x8f54585581b33d177af0f97c0b8d3a42 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0fbfa9e82e9d93a82d5fa4914a6e6c29 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x999f86162873b9efb425204ab5a801cc . "./code.cg2js/cg-outline.fasl")
(#x03104099e4c36adcb1cfac1725d7e583 . "./code.cg2js/cg-palette.fasl")
(#x7e79338220f18665662f60bd416158f1 . "./code.cg2js/cg-paren-matching.fasl")
(#x99ebd6537b764d82ef2569a319c0416a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x49da96dac9ca418a1c7917f924d48aa6 . "./code.cg2js/cg-picture-widget.fasl")
(#x2764c3399c0bb5759aa0cd6d3ec62a35 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x97110e7bc007c7d58f13a5247fcb5ef9 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x342ba032ce3ceea6d250aa74949b4151 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x892e15730872d205eb8e6ecf6be525d8 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x9ab2464655aef2b764435d74c68c3734 . "./code.cg2js/cg-pixmap.fasl")
(#x048537589cd1ce9ce3103fc01c7a0972 . "./code.cg2js/cg-player.fasl")
(#x00b84b98d66854279cd49d90b11053dc . "./code.cg2js/cg-plot-widget.fasl")
(#x5882cb385558f57ffde3c914b43f8656 . "./code.cg2js/cg-printing.fasl")
(#x3758ab922ee7fada23c2ff4c24ebdfcb . "./code.cg2js/cg-progress-indicator.fasl")
(#x9fd4605717478ee6e0d34ea6054eb3d5 . "./code.cg2js/cg-project-window.fasl")
(#x5eab00f4751ea3ff3352505eefb4fe45 . "./code.cg2js/cg-property.fasl")
(#x2143cc5fb90dc8d7691f336c9ef7b629 . "./code.cg2js/cg-radio-button.fasl")
(#x3a738934a6e9927c2de8c9054d50fefa . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x7380e19c95df8dc09732ea69f3bb4df2 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xe1493f6f83d76726c1ea536cb0216646 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xa1084b393c410c1f6f1b01604e4a0483 . "./code.cg2js/cg-rich-edit.fasl")
(#x1854a30a39ae823cec458e8e266bd7df . "./code.cg2js/cg-sample-file-menu.fasl")
(#x4ee5fbf77cf189aba7a1c7cea1885c1a . "./code.cg2js/cg-scaling-stream.fasl")
(#x449e9af0f14d231559dc955ee045709d . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3de6cdfe7b2c7e97a93b4360c3040cd0 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4c5755db3d970ab0b35b78ad801c2b25 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xfea97d6ee71302c5dbd56babf705ba01 . "./code.cg2js/cg-selected-object.fasl")
(#x40a5562f9c4f7f59a821e5eef7d3e393 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x63391bdbc215ff5c34ca3aab69bbc87c . "./code.cg2js/cg-split-bar.fasl")
(#xc6da3be4a50c977d21d57605d79f7641 . "./code.cg2js/cg-static-text.fasl")
(#x53b8ef8359e20d7a5d6f6f2d10b8b1fa . "./code.cg2js/cg-status-bar.fasl")
(#xff47070e712b6c897da10eb34b860b83 . "./code.cg2js/cg-string-dialog.fasl")
(#xba4c83906f4f532042e5a94b46bd4344 . "./code.cg2js/cg-tab-control.fasl")
(#x912cbf6cae670d2ecc1bc4929f41d845 . "./code.cg2js/cg-template-string.fasl")
(#x0213f82255411bc3f938ba97b82a736b . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x6e7e1d55a66d44cf3d074bfb9105347b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x50dcd70bbfc060d0ce26139cdf9e85d1 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xb2de885efb2db4d8ed8a0ee95d7f108c . "./code.cg2js/cg-text-or-combo.fasl")
(#xddd706a1f1e0254db7e034b68767067f . "./code.cg2js/cg-text-widget.fasl")
(#x927f9b3f6bc3e5c4bde0cd1f77e43950 . "./code.cg2js/cg-timer.fasl")
(#x46cd695ebde77b67ff8f75e2da7d2218 . "./code.cg2js/cg-toggling-widget.fasl")
(#x8feb81496a2b8e64487b24c5c9694a49 . "./code.cg2js/cg-toolbar.fasl")
(#x9db69d1642f23da39a600746ee2036bc . "./code.cg2js/cg-tooltip.fasl")
(#x22c73cc0b1ffc57419386563bfae4c82 . "./code.cg2js/cg-touch.fasl")
(#x13ee91b8a31a17fc552e25836bc208d6 . "./code.cg2js/cg-trackbar.fasl")
(#xab5275c3f30aa9dc43824a796f9c22f3 . "./code.cg2js/cg-tray.fasl")
(#x70233f9eb215cb5b9ef57366a946db0b . "./code.cg2js/cg-up-down-control.fasl")
(#xae0a8607da2897054db63cc2f5ced438 . "./code.cg2js/cg-utility-dialog.fasl")
(#x81beac08551775e5a24513019140e52c . "./code.cg2js/cg-web-browser-dde.fasl")
(#xbcd3319a42ee5c25d43291c997008a21 . "./code.cg2js/cg-web-browser.fasl")
(#xc989a43de1c117ccb3356f59ab4edd51 . "./code.cg2js/cg-wrap-string.fasl")
(#x343b7cb2136b58208442120cf368ee69 . "./code.cg2js/cg-yes-no-list.fasl")
(#x0e58cb62f0a8719b1c77ba16ad4918e6 . "./code.cg2js/cg.fasl")
(#x58353f95acb419c142aa53419e068f44 . "./code.cg2js/cg-yes-no-string.fasl")
(#x5b56ff33279246eac3e358ecd2955914 . "./code.idejs/ide-acache.fasl")
(#x4cc40dea3e92eac55b80bdcbc76fe8d5 . "./code.idejs/ide-apropos.fasl")
(#x5d61ba7582e7e2b95c291d60dbeafafb . "./code.idejs/ide-base.fasl")
(#x7c430ca3f525cc583f2b41a0d3d9407e . "./code.idejs/ide-class-browser.fasl")
(#xd1543df75d491aa1a1225a050d1c5cb9 . "./code.idejs/ide-class-grapher.fasl")
(#x0515f6fb26f086476e22481c93b6d121 . "./code.idejs/ide-clipboard.fasl")
(#x0ee225ca9fbbdb5e1e623c9e571d0c15 . "./code.idejs/ide-dev.fasl")
(#x51d224604fd41059b1ee2728d1df665d . "./code.idejs/ide-editor.fasl")
(#x61c4af68440daec57759ae638d4b439b . "./code.idejs/ide-find-definitions.fasl")
(#xf5c747db827ddf3814319ee8949eb809 . "./code.idejs/ide-find-in-files.fasl")
(#xe8f59c2c574e7f3faf0e75468d8788fd . "./code.idejs/ide-form.fasl")
(#x83d0358e841bb5aae5bd22a98e2fbc65 . "./code.idejs/ide-grapher.fasl")
(#x9e63e0846bc25b864abe57352801baca . "./code.idejs/ide-inspector.fasl")
(#xfe7313fb94de10eb79f75a53ad43f6f1 . "./code.idejs/ide-intro.fasl")
(#x0c3df3bed84ba1f607615a9f168dfd05 . "./code.idejs/ide-keyshort.fasl")
(#x158a3c129dd60dafa473fc5513fb8433 . "./code.idejs/ide-navigator.fasl")
(#xa33d06f63d2ddafd5db06b1248708482 . "./code.idejs/ide-package-list.fasl")
(#x848b91600964fdd3b0479ff2c7a4c8f5 . "./code.idejs/ide-patch-dialog.fasl")
(#x4bf94b9b164c75a73ec95e1f839b3ac4 . "./code.idejs/ide-process-browser.fasl")
(#x5ffa9f71eff955f3e5f44039bc363f21 . "./code.idejs/ide-project.fasl")
(#xc4135b13bc572b672c11229dee514bc0 . "./code.idejs/ide-trace.fasl")
(#x8d52240b3f20edd1d09c89a5de9222bc . "./code.idejs/ide-window-list.fasl")
(#xe8ab1875408b434d783c936c92a1f484 . "./code.idejs/ide.fasl")
(#xde049c9404b4ba293f0cbc6a4817dc96 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
